from dune.perftool.generation import (backend,iname,domain)
from dune.perftool.generation.loopy import transform
from dune.perftool.pdelab.localoperator import (determine_accumulation_space,
                                                boundary_predicates)

@iname
def sub_element_inames():
    name = "subel"
    domain(name, 4)
    return (name,)


@backend(interface="accum_insn", name='blockstructured')
def generate_accumulation_instruction(visitor, accterm, measure, subdomain_id):
    # When we do not do sumfactorization we do not split the test function
    assert(accterm.argument.expr is None)

    # Do the tree traversal to get a pymbolic expression representing this expression
    pymbolic_expr = visitor(accterm.term)

    # It may happen that an entire accumulation term vanishes. We do nothing in that case
    if pymbolic_expr == 0:
        return

    # Collect the lfs and lfs indices for the accumulate call
    test_lfs = determine_accumulation_space(accterm.term, 0, measure)

    # In the jacobian case, also determine the space for the ansatz space
    ansatz_lfs = determine_accumulation_space(accterm.term, 1, measure)

    # Collect the lfs and lfs indices for the accumulate call
    from dune.perftool.pdelab.argument import name_accumulation_variable
    accumvar = name_accumulation_variable((test_lfs.get_restriction() + ansatz_lfs.get_restriction()))

    predicates = boundary_predicates(accterm.term, measure, subdomain_id, visitor)

    rank = 1 if ansatz_lfs.lfs is None else 2

    from dune.perftool.pdelab.argument import PDELabAccumulationFunction
    from pymbolic.primitives import Call
    expr = Call(PDELabAccumulationFunction(accumvar, rank),
                (test_lfs.get_args() + ansatz_lfs.get_args() + (pymbolic_expr,))
                )

    from dune.perftool.generation import instruction
    from dune.perftool.options import option_switch
    quad_inames = visitor.interface.quadrature_inames()

    instruction(assignees=(),
                expression=expr,
                forced_iname_deps=frozenset(visitor.inames).union(frozenset(quad_inames)),
                forced_iname_deps_is_final=True,
                predicates=predicates
                )

    subelem_inames = sub_element_inames()

    from dune.perftool.sumfact.quadrature import nest_quadrature_loops
    transform(nest_quadrature_loops, subelem_inames)