from dune.perftool.generation import (backend,
                                      temporary_variable,
                                      domain)
from dune.perftool.pdelab.quadrature import (name_quadrature_points,
                                             quadrature_iname
                                             )
from dune.perftool.pdelab.geometry import world_dimension
from dune.perftool.blockstructured.accumulation import sub_element_inames
from dune.perftool.generation.loopy import instruction
from dune.perftool.options import get_option
import pymbolic.primitives as prim


@backend(interface="quad_pos", name='blockstructured')
def pymbolic_quadrature_position():
    quad_points = name_quadrature_points()
    quad_iname = quadrature_iname()
    from pymbolic.primitives import Subscript, Variable, Sum, Quotient

    qp_in_micro = Subscript(Variable(quad_points), (Variable(quad_iname,)))

    dim = world_dimension()
    shape_impl = ('fv',)
    qp_in_macro = 'macro_qp'
    temporary_variable(qp_in_macro, shape_impl=shape_impl, shape=(dim,))
    assignment_iname = 'assignment_iname'
    domain(assignment_iname, dim)

    expr = Subscript(qp_in_micro.aggregate, (qp_in_micro.index,) + (Variable(assignment_iname),))
    expr = Sum((expr, Variable(assignment_iname)))
    expr = Quotient(expr, get_option('number_of_blocks'))

    instruction(assignee=Subscript(Variable(qp_in_macro), (Variable(assignment_iname),)),
                expression=expr,
                forced_iname_deps=frozenset((assignment_iname,)),
                forced_iname_deps_is_final=True,
                tags=frozenset({'quad'})
                )

    return Variable(qp_in_macro)
